/**
 * This Class takes a GeoPosition and a callback in it's constructor
 * It will then evaluate the Position into an Object with a name etc which will require an Ajax call
 * If a callback is given this Class will call it when the Position is evaluated with an Instance of this object as first argument
 **/
function Waypoint(lon, lat, nominatimParser, gpsManager, index, map) {
  this.lon = lon;
  this.lat = lat;
  this.nominatimParser = nominatimParser;
  this.gpsManager = gpsManager;
  this.index = index;
  this.map = map;
  this.resultHtml = null;
  this.charCode = String.fromCharCode(97 + index).toUpperCase();
  this.evaluated = false;
}

Waypoint.prototype.evaluate = function() {
  var waypoint = this;
  return new Promise((resolve, reject) => {
    if (waypoint.lon !== undefined && waypoint.lat !== undefined) {
      var el = document.createElement("span");
      el.className = "marker";
      el.setAttribute("data-resultNumber", waypoint.index);
      el.innerHTML = waypoint.charCode;
      waypoint.marker = new maplibregl.Marker(el).setLngLat([
        waypoint.lon,
        waypoint.lat,
      ]);
      waypoint
        .positionToAdress()
        .then((data) => {
          waypoint.data = new NominatimParser(data);
          waypoint.evaluated = true;
          resolve(waypoint);
        })
        .catch((reason) => {
          reject(reason);
        });
    } else if (waypoint.nominatimParser !== undefined) {
      waypoint.lon = parseFloat(waypoint.nominatimParser.nominatimResult.lon);
      waypoint.lat = parseFloat(waypoint.nominatimParser.nominatimResult.lat);
      waypoint.data = waypoint.nominatimParser;
      waypoint.evaluated = true;
      var el = document.createElement("span");
      el.className = "marker";
      el.setAttribute("data-resultNumber", waypoint.index);
      el.innerHTML = waypoint.charCode;
      waypoint.marker = new maplibregl.Marker(el).setLngLat([
        waypoint.lon,
        waypoint.lat,
      ]);
      resolve(waypoint);
    } else if (waypoint.gpsManager !== undefined) {
      waypoint.gpsManager
        .getLocation()
        .then((location) => {
          waypoint.lon = location.coords.longitude;
          waypoint.lat = location.coords.latitude;
          var el = document.createElement("span");
          el.className = "marker";
          el.setAttribute("data-resultNumber", waypoint.index);
          el.innerHTML = waypoint.charCode;
          waypoint.marker = new maplibregl.Marker(el).setLngLat([
            waypoint.lon,
            waypoint.lat,
          ]);
          if (waypoint.index == 0) {
            waypoint.data = waypoint.gpsManager;
            waypoint.type = "gps";
            waypoint.evaluated = true;
            resolve(waypoint);
          } else {
            waypoint
              .positionToAdress()
              .then((data) => {
                waypoint.data = new NominatimParser(data);
                waypoint.evaluated = true;
                resolve(waypoint);
              })
              .catch((reason) => {
                reject(reason);
              });
          }
        })
        .catch((reason) => {
          reject(reason);
        });
    } else {
      reject("Format für Wegpunkt nicht definiert!");
    }
  });
};

Waypoint.prototype.changeIndex = function(newIndex) {
  this.index = newIndex;
  this.charCode = String.fromCharCode(97 + newIndex).toUpperCase();
  $(this.resultHtml)
    .find(".marker")
    .html(this.charCode);
  $(this.resultHtml).attr("data-index", newIndex);
  $(this.resultHtml)
    .find(".delete-waypoint")
    .attr("data-index", newIndex);
  this.marker.remove();
  var el = document.createElement("span");
  el.className = "marker";
  el.setAttribute("data-resultNumber", this.index);
  el.innerHTML = this.charCode;
  this.marker = new maplibregl.Marker(el).setLngLat([this.lon, this.lat]);
};

Waypoint.prototype.positionToAdress = function(errorCount) {
  errorCount = errorCount || 0;
  var pos = [this.lon, this.lat];
  var url = "/reverse/" + pos[0] + "/" + pos[1];
  const maxErrors = 5;

  // Start the ajax call
  var timeout = 10; // 10 seconds Timeout for this request

  var controller = new AbortController();
  var signal = controller.signal;

  var timeoutObj = setTimeout(() => controller.abort(), timeout * 1000);
  return new Promise((resolve, reject) => {
    if (errorCount >= maxErrors)
      return reject("Too many errors while fetching");

    fetch(url, { signal: signal, redirect: "manual", cache: "no-cache" })
      .then((response) => {
        clearTimeout(timeoutObj);
        if (response.status == 200) {
          response.json().then((data) => {
            if (data.error) {
              this.positionToAdress(errorCount + 1)
                .then(resolve)
                .catch(reject);
            } else {
              resolve(data);
            }
          });
        } else {
          this.positionToAdress(errorCount + 1)
            .then(resolve)
            .catch(reject);
        }
      })
      .catch((reason) => {
        this.positionToAdress(errorCount + 1)
          .then(resolve)
          .catch(reject);
      });
  });
};

Waypoint.prototype.getHtml = function() {
  if (this.evaluated) {
    if (this.resultHtml === null) {
      var description =
        typeof this.data.getHTMLAddressDetails === "function"
          ? this.data.getHTMLAddressDetails()
          : "Eigene Position";
      this.resultHtml = $(
        '\
			<li class="wp" data-index="' +
          this.index +
          '">\
				<div class="waypoint">\
					<div class="drag">\
						<img src="/img/anfasser.png" width="30px" alt="drag here" />\
					</div>\
					<div class="marker">\
						' +
          this.charCode +
          '\
					</div>\
					<div class="description">\
						' +
          description +
          '\
					</div>\
					<div class="delete-waypoint" data-index="' +
          this.index +
          '">\
						<span class="glyphicon glyphicon-trash"></span>\
					</div>\
				</div>\
			</li>\
			'
      );
    }
    return this.resultHtml;
  } else {
    return "Not Ready Yet";
  }
};
