function Map(type) {}

function InteractiveMap(
  long = 9.841943417968748,
  lat = 52.18082778659789,
  zoom = 8,
  updateMapPositionOnGps = true
) {
  Map.call(this);
  if (isNaN(long) || isNaN(lat) || isNaN(zoom)) {
    this.initPos = [9.841943417968748, 52.18082778659789];
    this.initZoom = 8;
  } else {
    this.initPos = [long, lat];
    this.initZoom = zoom;
  }

  // Remove downloaded areas from mobiles as the mechanic gets reworked
  this.removeDownloadedAreas();

  this.updateMapPositionOnGps = updateMapPositionOnGps;
  // Initialize the Map With Controls to change the view
  this.map = this.initMap();
  this.module = null;
  this.GpsManager = new GpsManager(this);
  // Initialize the Positions gathering on click on the Map
  this.reversePositionManager = new ReversePositionManager(this); // This is the Overlay that displays informations about a position where the user has clicked.
}

InteractiveMap.prototype = Object.create(Map.prototype);
InteractiveMap.prototype.constructor = InteractiveMap;

InteractiveMap.prototype.initMap = function() {
  maplibregl.setRTLTextPlugin(
    "https://tileserver.metager.de/maplibre-gl-rtl-text.js"
  );
  var map = new maplibregl.Map({
    container: "map",
    style:
      "https://tileserver.metager.de/styles/osm-bright/style.json?optimize=true",
    hash: false,
    center: this.initPos,
    zoom: this.initZoom,
    scrollZoom: false,
  });

  map.addControl(new maplibregl.NavigationControl(), "bottom-left");
  map.addControl(new maplibregl.ScaleControl(), "bottom-right");

  /**
   * This is our custom scrollwheel function
   * It allows faster and cummulated zooms
   */
  var caller = this;
  var timeout = undefined;
  var zoomChange = 0;
  map.on("zoomend", function(event) {
    if (typeof event.scrollZoom == "boolean" && event.scrollZoom) {
      if (typeof timeout != "undefined") {
        clearTimeout(timeout);
        timeout = undefined;
      }
      timeout = setTimeout(function() {
        zoomChange = 0;
      }, 100);
    }
  });
  $("#map").bind("wheel", function(event) {
    console.log("wheel");
    var delta = event.originalEvent.deltaY;
    if (delta < 0) {
      if (zoomChange >= 0) zoomChange += 0.5;
      else zoomChange = 0.5;
    } else {
      if (zoomChange <= 0) zoomChange -= 0.5;
      else zoomChange = -0.5;
    }
    var newZoom = Math.min(caller.map.getZoom() + zoomChange, 22);
    newZoom = Math.max(1, newZoom);

    map.easeTo(
      {
        zoom: newZoom,
        around: map.unproject([
          event.originalEvent.clientX,
          event.originalEvent.clientY,
        ]),
      },
      { scrollZoom: true }
    );
  });
  return map;
};

InteractiveMap.prototype.scrollZoomHandler = function(event) {
  console.log(event);
};

InteractiveMap.prototype.switchModule = function(name, args) {
  if (this.module !== null) {
    // Every Module must implement this method for deinitialization
    this.module.exit();
    this.module = null;
  }
  switch (name) {
    case "search":
      // The search Module can be started with or without a search term
      if (typeof args == "string") {
        this.module = new SearchModule(this, args);
      } else {
        this.module = new SearchModule(this);
      }
      break;
    case "route-finding":
      this.module = new RouteFinder(this, args.waypoints, args.vehicle);
      break;
    case "navigatiion":
      break;
    case "offline-karten":
      this.module = new OfflineModule(this);
      break;
    case "navigation":
      this.module = new NavigationModule(this, args);
    default:
      return;
  }
};

InteractiveMap.prototype.removeDownloadedAreas = function() {
  /**
   * Removes areas that might have been downloaded by our old app.
   * This Feature will be reworked and we want to clean up before the new app comes
   */
  if (
    typeof android != "undefined" &&
    typeof android.getDownloadedAreas == "function"
  ) {
    var downloadedAreas = android.getDownloadedAreas(); // JSON.parse(android.getDownloadedAreas());
    downloadedAreas = JSON.parse(downloadedAreas);
    if (Object.keys(downloadedAreas).length > 0) {
      $.each(downloadedAreas, function(index, value) {
        android.removeArea(index);
      });
      alert(
        "Offline Karten wurden für Wartungsarbeiten deaktiviert. Sie stehen bald wieder zur Verfügung!"
      );
    }
  }
};

function StaticMap() {
  Map.call(this);
}
